@@ -17,6 +17,7 @@ __date__ = "2/4/13"
 import subprocess
 import os
 import shutil
+import math
 
 from pymatgen.util.io_utils import zopen
 from pymatgen.io.vaspio.vasp_input import VaspInput
@@ -44,8 +45,8 @@ class VaspJob(Job, MSONable):
 
     def __init__(self, vasp_command, output_file="vasp.out", suffix="",
                  final=True, gzipped=False, backup=True,
-                 default_vasp_input_set=MITVaspInputSet(),
-                 settings_override=None):
+                 default_vasp_input_set=MITVaspInputSet(), auto_npar=True,
+                 auto_gamma=True, settings_override=None):
         """
         This constructor is necessarily complex due to the need for
         flexibility. For standard kinds of runs, it's often better to use one
@@ -79,6 +80,17 @@ class VaspJob(Job, MSONable):
                 input files for the run. If the directory already
                 contain a full set of VASP input files,
                 this input is ignored. Defaults to the MITVaspInputSet.
+            auto_npar:
+                Whether to automatically tune NPAR to be sqrt(number of
+                cores) as recommended by VASP for DFT calculations.
+                Generally, this results in significant speedups. Defaults to
+                True. Set to False for HF, GW and RPA calculations.
+            auto_gamma:
+                Whether to automatically check if run is a Gamma 1x1x1 run,
+                and whether a Gamma optimized version of VASP exists with
+                ".gamma" appended to the name of the VASP executable. If so,
+                run the gamma optimized version of VASP instead of regular
+                VASP.
             settings_override:
                 An ansible style list of dict to override changes. For example,
                 to set ISTART=1 for subsequent runs and to copy the CONTCAR
@@ -96,6 +108,8 @@ class VaspJob(Job, MSONable):
         self.default_vis = default_vasp_input_set
         self.suffix = suffix
         self.settings_override = settings_override
+        self.auto_npar = auto_npar
+        self.auto_gamma = auto_gamma
 
     def setup(self):
         files = os.listdir(".")
@@ -117,6 +131,22 @@ class VaspJob(Job, MSONable):
             for f in VASP_INPUT_FILES:
                 shutil.copy(f, "{}.orig".format(f))
 
+        if self.auto_npar:
+            try:
+                vi = VaspInput.from_directory(".")
+                incar = vi["INCAR"]
+                #Only optimized NPAR for non-HF and non-RPA calculations.
+                if (not incar.get("LHFCALC")) and (not incar.get("LRPA")):
+                    import multiprocessing
+                    ncores = multiprocessing.cpu_count()
+                    for npar in range(int(round(math.sqrt(ncores))), ncores):
+                        if ncores % npar == 0:
+                            incar["NPAR"] = npar
+                            break
+                    incar.write_file("INCAR")
+            except:
+                pass
+
         if self.settings_override is not None:
             vi = VaspInput.from_directory(".")
             m = Modder([FileActions, DictActions])
@@ -132,8 +162,16 @@ class VaspJob(Job, MSONable):
                 vi[f].write_file(f)
 
     def run(self):
+        cmd = list(self.vasp_command)
+        if self.auto_gamma:
+            vi = VaspInput.from_directory(".")
+            kpts = vi["KPOINTS"]
+            if kpts.style == "Gamma" and tuple(kpts.kpts[0]) == (1, 1, 1):
+                if os.path.exists(cmd[-1] + ".gamma"):
+                    cmd[-1] += ".gamma"
+
         with open(self.output_file, 'w') as f:
-            return subprocess.Popen(self.vasp_command, stdout=f)
+            return subprocess.Popen(cmd, stdout=f)
 
     def postprocess(self):
         for f in VASP_OUTPUT_FILES + [self.output_file]:
@@ -181,6 +219,7 @@ class VaspJob(Job, MSONable):
                  output_file=self.output_file, suffix=self.suffix,
                  final=self.final, gzipped=self.gzipped, backup=self.backup,
                  default_vasp_input_set=self.default_vis.to_dict,
+                 auto_npar=self.auto_npar, auto_gamma=self.auto_gamma,
                  settings_override=self.settings_override
                  )
         d["@module"] = self.__class__.__module__
@@ -193,8 +232,8 @@ class VaspJob(Job, MSONable):
         return VaspJob(
             vasp_command=d["vasp_command"], output_file=d["output_file"],
             suffix=d["suffix"], final=d["final"], gzipped=d["gzipped"],
-            backup=d["backup"], default_vasp_input_set=vis,
-            settings_override=d["settings_override"])
+            backup=d["backup"], default_vasp_input_set=vis, auto_npar=d['auto_npar'],
+            auto_gamma=d['auto_gamma'], settings_override=d["settings_override"])
 
 
 def gzip_directory(path):
